﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Reflection.Metadata.Ecma335;
using System.Text;
using System.Threading.Tasks;
using ProjectLinkor.Entities;

namespace ProjectLinkor.Drawnings
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawingLinkor : DrawingShip
    {
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="rocket">Признак наличия обвеса</param>
        /// <param name="gun">Признак наличия антикрыла</param>
        /// <param name="sportLine">Признак наличия гоночной полосы</param>

        public DrawingLinkor(int speed, double weight, Color bodyColor, Color additionalColor, bool rocket, bool gun) : base(110, 70)
        {
            EntityShip = new EntityLinkor(speed, weight, bodyColor, additionalColor, rocket, gun);
        }
        public DrawingLinkor(EntityLinkor? linkor) : base(132, 65)
        {
            if (linkor == null) return;
            EntityShip = new EntityLinkor(linkor.Speed, linkor.Weight, linkor.BodyColor,
                linkor.AdditionalColor, linkor.Rocket, linkor.Gun);
        }
        public override void DrawTransport(Graphics g)
        {
            if (EntityShip == null || EntityShip is not EntityLinkor linkor || !_startPosX.HasValue || !_startPosY.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new SolidBrush(linkor.AdditionalColor);

            Point[] body2 = { new Point(_startPosX.Value + 50, _startPosY.Value + 10),
                new Point(_startPosX.Value + 60, _startPosY.Value + 15), new Point(_startPosX.Value +50, _startPosY.Value + 20) };
            Point[] body3 = { new Point(_startPosX.Value + 50, _startPosY.Value + 60),
                new Point(_startPosX.Value + 60, _startPosY.Value + 65), new Point(_startPosX.Value + 50, _startPosY.Value + 70) };
            if (linkor.Rocket)
            {
                //ракеты
                g.FillRectangle(additionalBrush, _startPosX.Value + 30, _startPosY.Value + 10, 20, 10);
                g.FillRectangle(additionalBrush, _startPosX.Value + 30, _startPosY.Value + 60, 20, 10);
                g.FillPolygon(additionalBrush, body2);
                g.FillPolygon(additionalBrush, body3);

            }
            _startPosX += 5;
            _startPosY += 20;
            base.DrawTransport(g);
            _startPosX -= 5;
            _startPosY -= 20;

            if (linkor.Gun)
            {
                //пушка
                g.FillEllipse(additionalBrush, _startPosX.Value + 80, _startPosY.Value + 35, 10, 10);
                g.FillRectangle(additionalBrush, _startPosX.Value + 55, _startPosY.Value + 35, 30, 10);
            }
        }
    
    }
}
