﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectLinkor.Entities;

namespace ProjectLinkor.Drawnings;

public static class ExtentionDrawingShip
{
    /// <summary>
    /// Разделитель для записи информации по объекту в файл
    /// </summary>
    private static readonly string _separatorForObject = ":";
    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static DrawingShip? CreateDrawingShip(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityShip? ship = EntityLinkor.CreateEntityLinkor(strs);

        if (ship != null)
        {
            return new DrawingLinkor((EntityLinkor)ship);
        }
        ship = EntityShip.CreateEntityShip(strs);
        if (ship != null)
        {
            return new DrawingShip(ship);
        }
        return null;
    }
    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningCar">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this DrawingShip drawingShip)
    {
        string[]? array = drawingShip?.EntityShip?.GetStringRepresentation();
        if (array == null)
        {
            return string.Empty;
        }
        return string.Join(_separatorForObject, array);
    }

}
