﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectLinkor.Entities
{
    /// <summary>
    /// Класс-сущность "Линкор"
    /// </summary>
    public class EntityLinkor : EntityShip
    {
        public Color AdditionalColor { get; private set; }
        public void SetAdditionalColor(Color additionalColor)
        {
            AdditionalColor = additionalColor;
        }
        /// <summary>
        /// Признак (опция) наличия ракеты
        /// </summary>
        public bool Rocket { get; private set; }
        /// <summary>
        /// Признак (опция) наличия оружия
        /// </summary>
        public bool Gun { get; private set; }

        /// <summary>
        /// Шаг перемещения линкора
        /// </summary>
        public double Step => Speed * 100 / Weight;
        /// <summary>
        /// Инициализация полей объекта-класса спортивного автомобиля
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес автомобиля</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="rocket">Признак наличия обвеса</param>
        /// <param name="gun">Признак наличия антикрыла</param>
        /// <param name="sportLine">Признак наличия гоночной полосы</param>
        public EntityLinkor(int speed, double weight, Color bodyColor, Color
        additionalColor, bool rocket, bool gun) : base(speed, weight, bodyColor)
        {
            AdditionalColor = additionalColor;
            Rocket = rocket;
            Gun = gun;
        }

        /// <summary>
        /// Получение строк со значениями свойств продвинутого объекта класса-сущности
        /// </summary>
        /// <returns></returns>
        public override string[] GetStringRepresentation()
        {
            return new[] { nameof(EntityLinkor), Speed.ToString(), Weight.ToString(),
            BodyColor.Name, AdditionalColor.Name, Rocket.ToString(), Gun.ToString()};
        }
        /// <summary>
        /// Создание продвинутого объекта из массива строк
        /// </summary>
        /// <param name="strs"></param>
        /// <returns></returns>
        public static EntityLinkor? CreateEntityLinkor(string[] strs)
        {
            if (strs.Length != 7 || strs[0] != nameof(EntityLinkor))
            {
                return null;
            }
            return new EntityLinkor(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]),
            Color.FromName(strs[3]), Color.FromName(strs[4]), Convert.ToBoolean(strs[5]),
            Convert.ToBoolean(strs[6]));
        }

    }
}
