﻿using ProjectLinkor.Drawnings;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TrackBar;
using ProjectLinkor.Entities;

namespace ProjectLinkor;

/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormShipConfig : Form
{
    /// <summary>
    /// Объект - прорисовка автомобиля
    /// </summary>
    private DrawingShip? _ship;

    /// <summary>
    /// Событие для передачи объекта
    /// </summary>
    private event Action<DrawingShip>? _shipDelegate;
    /// <summary>
    /// Конструктор
    /// </summary>
    public FormShipConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGrey.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;
        //TODO
        buttonCancel.Click += (sender, e) => Close();
    }
    /// <summary>
    /// Привязка внешнего метода и событие
    /// </summary>
    /// <param name="shipDelegate"></param>
    public void AddEvent(Action<DrawingShip>? shipDelegate)
    {
        _shipDelegate += shipDelegate;
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _ship?.SetPictureSize(pictureBoxObject.Width,
        pictureBoxObject.Height);
        _ship?.SetPosition(50, 40);
        _ship?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }
    /// <summary>
    /// Передаем информацию при нажатии на Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }
    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }
    /// <summary>
    /// Действия при приеме перетаскиваемой информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _ship = new DrawingShip((int)numericUpDownSpeed.Value,
                (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _ship = new DrawingLinkor((int)numericUpDownSpeed.Value,
                (double)numericUpDownWeight.Value,
                Color.White, Color.Black, checkBoxRocket.Checked, checkBoxGun.Checked);
                break;
        }
        DrawObject();
    }
    /// <summary>
    /// Передача информации при нажатии на Panel
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        (sender as Control)?.DoDragDrop((sender as Control)?.BackColor!, DragDropEffects.Move | DragDropEffects.Copy);
    }
    /// <summary>
    /// Передача объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        if (_ship != null)
        {
            _shipDelegate?.Invoke(_ship);
            Close();
        }
    }
    private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_ship == null) return;
        _ship.EntityShip?.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
        DrawObject();
    }

    private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color))) e.Effect = DragDropEffects.Copy;
        else e.Effect = DragDropEffects.None;
    }

    private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_ship?.EntityShip is EntityLinkor _linkor)
            _linkor?.SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        DrawObject();
    }

    private void labelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color))) e.Effect = DragDropEffects.Copy;
        else e.Effect = DragDropEffects.None;
    }

}
