﻿using ProjectLinkor.Drawnings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectLinkor.MovementStrategy;
/// <summary>
/// Класс-реализация IMoveableObject с использованием DrawingShip
/// </summary>
public class MoveableShip : IMoveableObject 
{
    /// <summary>
    /// Поле-объект класса DrawningCar или его наследника
    /// </summary>
    private readonly DrawingShip? _ship = null;
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="car">Объект класса DrawningCar</param>
    public MoveableShip(DrawingShip ship)
    {
        _ship = ship;
    }

    public ObjectParametrs? GetObjectPosition
    {
        get
        {
            if (_ship == null || _ship.EntityShip == null ||
            !_ship.GetPosX.HasValue || !_ship.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParametrs(_ship.GetPosX.Value, _ship.GetPosY.Value, _ship.GetWidth, _ship.GetHeight);
        }
    }
    public int GetStep => (int)(_ship?.EntityShip?.Step ?? 0);
    public bool TryMoveObject(MovementDirection direction)
    {
        if (_ship == null || _ship.EntityShip == null)
        {
            return false;
        }
        return _ship.MoveTransport(GetDirectionType(direction));
    }


    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}
